"use strict";
var __awaiter = (this && this.__awaiter) || function (thisArg, _arguments, P, generator) {
    function adopt(value) { return value instanceof P ? value : new P(function (resolve) { resolve(value); }); }
    return new (P || (P = Promise))(function (resolve, reject) {
        function fulfilled(value) { try { step(generator.next(value)); } catch (e) { reject(e); } }
        function rejected(value) { try { step(generator["throw"](value)); } catch (e) { reject(e); } }
        function step(result) { result.done ? resolve(result.value) : adopt(result.value).then(fulfilled, rejected); }
        step((generator = generator.apply(thisArg, _arguments || [])).next());
    });
};
var __generator = (this && this.__generator) || function (thisArg, body) {
    var _ = { label: 0, sent: function() { if (t[0] & 1) throw t[1]; return t[1]; }, trys: [], ops: [] }, f, y, t, g;
    return g = { next: verb(0), "throw": verb(1), "return": verb(2) }, typeof Symbol === "function" && (g[Symbol.iterator] = function() { return this; }), g;
    function verb(n) { return function (v) { return step([n, v]); }; }
    function step(op) {
        if (f) throw new TypeError("Generator is already executing.");
        while (_) try {
            if (f = 1, y && (t = op[0] & 2 ? y["return"] : op[0] ? y["throw"] || ((t = y["return"]) && t.call(y), 0) : y.next) && !(t = t.call(y, op[1])).done) return t;
            if (y = 0, t) op = [op[0] & 2, t.value];
            switch (op[0]) {
                case 0: case 1: t = op; break;
                case 4: _.label++; return { value: op[1], done: false };
                case 5: _.label++; y = op[1]; op = [0]; continue;
                case 7: op = _.ops.pop(); _.trys.pop(); continue;
                default:
                    if (!(t = _.trys, t = t.length > 0 && t[t.length - 1]) && (op[0] === 6 || op[0] === 2)) { _ = 0; continue; }
                    if (op[0] === 3 && (!t || (op[1] > t[0] && op[1] < t[3]))) { _.label = op[1]; break; }
                    if (op[0] === 6 && _.label < t[1]) { _.label = t[1]; t = op; break; }
                    if (t && _.label < t[2]) { _.label = t[2]; _.ops.push(op); break; }
                    if (t[2]) _.ops.pop();
                    _.trys.pop(); continue;
            }
            op = body.call(thisArg, _);
        } catch (e) { op = [6, e]; y = 0; } finally { f = t = 0; }
        if (op[0] & 5) throw op[1]; return { value: op[0] ? op[1] : void 0, done: true };
    }
};
Object.defineProperty(exports, "__esModule", { value: true });
exports.Action = void 0;
/**
 * Represents a single step in an event.
 */
var Action = /** @class */ (function () {
    function Action(executionFn, compensation, retryFn) {
        this.executionFn = executionFn;
        this.compensation = compensation;
        this.retryFn = retryFn;
    }
    /**
     * Create a new Action
     * @param executionFn Function to execute
     * @param compensation Function to undo this action's changes.
     * @param retryFn Function that indicates if to retry execution or not.
     * @returns A new action with the same input and output as `executionFn`
     */
    Action.new = function (executionFn, compensation, retryFn) {
        return new Action(executionFn, compensation, retryFn);
    };
    /**
     * Execute this action and subsequent ones recursively.
     * @param data Input data for the execution function
     * @returns If the final action in an event, returns a Promise of the output of `executionFn`. Otherwise, returns the next action's execution.
     */
    Action.prototype.execute = function (data) {
        return __awaiter(this, void 0, void 0, function () {
            var newData, err_1, _a;
            return __generator(this, function (_b) {
                switch (_b.label) {
                    case 0:
                        _b.trys.push([0, 2, , 8]);
                        return [4 /*yield*/, this.executionFn(data)];
                    case 1:
                        newData = _b.sent();
                        return [3 /*break*/, 8];
                    case 2:
                        err_1 = _b.sent();
                        _a = this.retryFn;
                        if (!_a) return [3 /*break*/, 4];
                        return [4 /*yield*/, this.retryFn(err_1)];
                    case 3:
                        _a = (_b.sent());
                        _b.label = 4;
                    case 4:
                        if (!_a) return [3 /*break*/, 6];
                        return [4 /*yield*/, this.execute(data)];
                    case 5: return [2 /*return*/, _b.sent()];
                    case 6: throw err_1;
                    case 7: return [3 /*break*/, 8];
                    case 8: return [4 /*yield*/, this.executeNext(data, newData)];
                    case 9: return [2 /*return*/, _b.sent()];
                }
            });
        });
    };
    Action.prototype.executeNext = function (data, newData) {
        return __awaiter(this, void 0, void 0, function () {
            var err_2;
            return __generator(this, function (_a) {
                switch (_a.label) {
                    case 0:
                        if (!this.next) return [3 /*break*/, 6];
                        _a.label = 1;
                    case 1:
                        _a.trys.push([1, 3, , 6]);
                        return [4 /*yield*/, this.next.execute(newData)];
                    case 2: return [2 /*return*/, _a.sent()];
                    case 3:
                        err_2 = _a.sent();
                        if (!this.compensation) return [3 /*break*/, 5];
                        return [4 /*yield*/, this.compensation(newData, data, err_2)];
                    case 4:
                        _a.sent();
                        _a.label = 5;
                    case 5: throw err_2;
                    case 6: return [2 /*return*/, newData];
                }
            });
        });
    };
    /**
     * Set the next action to be executed (only once).
     * @param action Next action to be executed. Must have the same input as this action's output.
     * @returns Itself
     */
    Action.prototype.setNext = function (action) {
        if (this.next) {
            throw new Error('Next action already exists.');
        }
        this.next = action;
        return this;
    };
    /**
     * Function to run if subsequent actions fail.
     * @param compensation Function that receives this action's output, input, and the thrown error.
     * @returns Itself
     */
    Action.prototype.compensate = function (compensation) {
        if (this.compensation) {
            throw new Error('Compensation already exists.');
        }
        this.compensation = compensation;
        return this;
    };
    /**
     * Set the function that decides if to retry the execution.
     * @param retryFn Function that recieves the thrown error and returns a boolean.
     * @returns Itself
     */
    Action.prototype.retry = function (retryFn) {
        if (this.retryFn) {
            throw new Error('Retry function already exists.');
        }
        this.retryFn = retryFn;
        return this;
    };
    return Action;
}());
exports.Action = Action;
