import { Action } from './actions';
/**
 * Set of actions that make up a transactional operation.
 */
export declare class Eventra<I = void, O = void> {
    private actions;
    private constructor();
    /**
     * Create a new Event.
     * @param action Event's first operation.
     * @returns New Event with the same input and output as the action.
     */
    static new<U, V>(action: Action<U, V>): Eventra<U, V>;
    /**
     * Add a new action to an event.
     * @param action Action with the same input as the event's output
     * @returns New event with the same input as the previous one, with the new action's output.
     */
    use<V>(action: Action<O, V>): Eventra<I, V>;
    /**
     * Execute the event's actions, in order. Throws any error thrown within said actions.
     * @param data Data needed for the first action.
     * @returns Last action's output.
     */
    execute(data: I): Promise<O>;
}
