/**
 * Options for custom retry function.
 */
interface RetryOptions {
    /**
     * Number of times to retry. Default 1.
     */
    amount?: number;
    /**
     * Time in ms to wait before retrying. Run before custom function. Default 0.
     */
    delay?: number;
    /**
     * Function which recieves an error and returns a boolean, or a promise of a boolean. Default `() => true`.
     */
    custom?: (err?: Error) => boolean | Promise<boolean>;
}
/** Helper class for making custom retry functions. */
export declare class Retry {
    private static sleep;
    /**
     * Creates a new retry function.
     * @param options Options for the retry function with the following (optional) properties:
     * @returns New retry function.
     */
    static new(options: RetryOptions): (err?: Error | undefined) => Promise<boolean>;
}
export {};
